<?php
// Own namespace (watch out: core)
namespace CoreFramework\EntryPoint;

// Import framework stuff
use CoreFramework\Configuration\FrameworkConfiguration;
use CoreFramework\Factory\ObjectFactory;
use CoreFramework\Helper\Application\ApplicationHelper;
use CoreFramework\Loader\ClassLoader;
use CoreFramework\Generic\FrameworkException;

/**
 * The main class with the entry point to the whole application. This class
 * "emulates" Java's entry point call. Additionally it covers local
 * variables from outside access to prevent possible attacks on uninitialized
 * local variables.
 *
 * But good little boys and girls would always initialize their variables... ;-)
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2015 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
final class ApplicationEntryPoint {
	/**
	 * Core path
	 */
	private static $corePath = '';

	/**
	 * The application's emergency exit
	 *
	 * @param	$message		The optional message we shall output on exit
	 * @param	$code			Error code from exception
	 * @param	$extraData		Extra information from exceptions
	 * @param	$silentMode		Whether silent mode is turned on
	 * @return	void
	 * @todo	This method is old code and needs heavy rewrite and should be moved to ApplicationHelper
	 */
	public static final function app_exit ($message = '', $code = FALSE, $extraData = '', $silentMode = FALSE) {
		// Is this method already called?
		if (isset($GLOBALS['app_die_called'])) {
			// Then output the text directly
			exit($message);
		} // END - if

		// This method shall not be called twice
		$GLOBALS['app_die_called'] = TRUE;

		// Is a message set?
		if (empty($message)) {
			// No message provided
			$message = 'No message provided.';
		} // END - if

		// Get config instance
		$configInstance = FrameworkConfiguration::getSelfInstance();

		// Do we have debug installation?
		if (($configInstance->getConfigEntry('product_install_mode') == 'productive') || ($silentMode === TRUE)) {
			// Abort here
			exit();
		} // END - if

		// Get some instances
		$tpl = FrameworkConfiguration::getSelfInstance()->getConfigEntry('html_template_class');
		$languageInstance = LanguageSystem::getSelfInstance();

		// Initialize template instance here to avoid warnings in IDE
		$templateInstance = NULL;

		// Get response instance
		$responseInstance = ApplicationHelper::getSelfInstance()->getResponseInstance();

		// Is the template engine loaded?
		if ((class_exists($tpl)) && (is_object($languageInstance))) {
			// Use the template engine for putting out (nicer look) the message
			try {
				// Get the template instance from our object factory
				$templateInstance = ObjectFactory::createObjectByName($tpl);
			} catch (FrameworkException $e) {
				exit(sprintf("[Main:] Could not initialize template engine for reason: <span class=\"exception_reason\">%s</span>",
					$e->getMessage()
				));
			}

			// Get and prepare backtrace for output
			$backtraceArray = debug_backtrace();
			$backtrace = '';
			foreach ($backtraceArray as $key => $trace) {
				// Set missing array elements
				if (!isset($trace['file'])) {
					$trace['file'] = __FILE__;
				} // END - if
				if (!isset($trace['line'])) {
					$trace['line'] = __LINE__;
				} // END - if
				if (!isset($trace['args'])) {
					$trace['args'] = array();
				} // END - if

				// Add the traceback path to the final output
				$backtrace .= sprintf("<span class=\"backtrace_file\">%s</span>:%d, <span class=\"backtrace_function\">%s(%d)</span><br />\n",
					basename($trace['file']),
					$trace['line'],
					$trace['function'],
					count($trace['args'])
				);
			} // END - foreach

			// Init application instance
			$applicationInstance = NULL;

			// Is the class there?
			if (class_exists('CoreFramework\Helper\Application\ApplicationHelper')) {
				// Get application instance
				$applicationInstance = ApplicationHelper::getSelfInstance();

				// Assign application data
				$templateInstance->assignApplicationData($applicationInstance);
			} // END - if

			// We only try this
			try {
				// Assign variables
				$templateInstance->assignVariable('message', $message);
				$templateInstance->assignVariable('code', $code);
				$templateInstance->assignVariable('extra', $extraData);
				$templateInstance->assignVariable('backtrace', $backtrace);
				$templateInstance->assignVariable('total_includes', ClassLoader::getSelfInstance()->getTotal());
				$templateInstance->assignVariable('total_objects', ObjectFactory::getTotal());
				$templateInstance->assignVariable('title', $languageInstance->getMessage('emergency_exit_title'));

				// Load the template
				$templateInstance->loadCodeTemplate('emergency_exit');

				// Compile the template
				$templateInstance->compileTemplate();

				// Compile all variables
				$templateInstance->compileVariables();

				// Transfer data to response
				$templateInstance->transferToResponse($responseInstance);

				// Flush the response
				$responseInstance->flushBuffer();
			} catch (FileNotFoundException $e) {
				// Even the template 'emergency_exit' wasn't found so output both message
				exit($message . ', exception: ' . $e->getMessage());
			}

			// Good bye...
			exit();
		} else {
			// Output message and die
			exit(sprintf('[Main:] Emergency exit reached: <span class="emergency_span">%s</span>',
				$message
			));
		}
	}

	/**
	 * Determines the correct absolute path for all includes only once per run.
	 * Other calls of this method are being "cached".
	 *
	 * @return	$corePath	Base path (core) for all includes
	 */
	protected static final function detectCorePath () {
		// Is it not set?
		if (empty(self::$corePath)) {
			// Auto-detect our core path
			self::$corePath = str_replace("\\", '/', dirname(__FILE__));
		} // END - if

		// Return it
		return self::$corePath;
	}

	/**
	 * The framework's main entry point. This class isolates some local
	 * variables which shall not become visible to outside because of security
	 * concerns. This is done here to "emulate" the well-known entry point in
	 * Java.
	 *
	 * @return	void
	 */
	public static final function main () {
		// Load config file, this provides $cfg
		require(self::detectCorePath() . '/inc/config.php');

		// Get a new configuration instance
		$cfg = FrameworkConfiguration::getSelfInstance();

		// Load bootstrap class
		require($cfg->getConfigEntry('base_path') . 'inc/bootstrap/class_BootstrapFramework.php');

		// ----- Below is deprecated -----

		// Load all include files
		require($cfg->getConfigEntry('base_path') . 'inc/includes.php');

		// Include the application selector
		require($cfg->getConfigEntry('base_path') . 'inc/selector.php');
	}
}

// Developer mode active? Comment out if no dev!
define('DEVELOPER', TRUE);

// Log all exceptions (only debug! This option can create large error logs)
//define('LOG_EXCEPTIONS', TRUE);

//xdebug_start_trace();

// Call above main() method
ApplicationEntryPoint::main();
